#include "Camera.h"

/*-------------------------------------------------------
* update the cameras position - used by both Camera::turn()
* and Camera::move()
---------------------------------------------------------*/
void Camera::updatePosition(){
	float cs = cos(_xAngle); // hold the cos of the xAngle
	float sn = sin(_xAngle); // hold the sin of the xAngle
	float cs2 = cos(_yAngle); // hold the cos of the -yAngle
	float sn2 = sin(_yAngle); // hold the sin of the -yAngle

	_lookAt._x = cs*cs2 + _eye._x; // set the _lookAt Vector3d relative to the _eye Vector3d
	_lookAt._y = sn2 + _eye._y;
	_lookAt._z = sn*cs2 + _eye._z;

	_up._x = cs*-sn2; // rotate the _up Vector3d so it's in the right pos
	_up._y = cs2;	  // NOTE: _up Vector3d only needs to change when the pitch changes
	_up._z = sn*-sn2;
}

/*---------------------------------------
* set the matrix - uses gluLookAt
-----------------------------------------*/
void Camera::setMatrix() const{
	//load model view matrix with existing camera values
	gluLookAt(_eye._x, _eye._y, _eye._z, _lookAt._x, _lookAt._y, _lookAt._z, _up._x, _up._y, _up._z); // load the matrix
}

/*------------------------------------------------------------
* set the camera position - similar to gluLookAt - uses Vector3d
--------------------------------------------------------------*/
void Camera::setCameraPos(const Vector3d eye){
	//set the initial positions of the eye, lookat, and up vectors
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	_eye.set(eye);
}

/*----------------------------------------------------------------------------------
* overloaded setCameraPos function that takes the raw coords instead of Vector3d
------------------------------------------------------------------------------------*/
void Camera::setCameraPos(const double eyex, const double eyey, const double eyez){
	//set the initial positions of the eye, lookat, and up vectors
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	_eye.set(eyex, eyey, eyez);
}

/*-------------------------------------------------------------------
* set up the camera - uses gluPerspective
---------------------------------------------------------------------*/
void Camera::setCameraShape(const double fov, const double aspect, const double nearDist, const double farDist){
	_dFov = fov; // field of view
	_dAspect = aspect; // aspect ratio, eg 16:9
	_dNearDist = nearDist; // near distance (z)
	_dFarDist = farDist; // far distance (z)

	// set up the viewport
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(_dFov, _dAspect, _dNearDist, _dFarDist);
}

/*------------------------------------------------------------------------
* turn the camera. Gets its angles from the x and y of the mouse position
--------------------------------------------------------------------------*/
void Camera::turn(const float xAngle, const float yAngle){
	_xAngle = xAngle;
	_yAngle = yAngle;
	updatePosition(); //update the lookAt and up Vector3d's
}

/*---------------------------------------------
* move the camera along the eye/lookAt vector
-----------------------------------------------*/
void Camera::move(const float amount){
	double xdiff = _lookAt._x - _eye._x;
	double zdiff = _lookAt._z - _eye._z;
	_eye._x -= xdiff*amount;
	_eye._z -= zdiff*amount;
	updatePosition(); //update the lookAt and up Vector3d's
}

/*---------------------------------------------
* move the camera to a specified Vector3d
-----------------------------------------------*/
void Camera::move(const Vector3d& pos){
	_eye.set(pos);
	updatePosition(); //update the lookAt and up Vector3d's
}

/*---------------------------------------------------------------------------------
* calculate next move, and return the coords - but don't move there yet
-----------------------------------------------------------------------------------*/
Vector3d Camera::calcNextMove(const float amount){
	double xdiff = _lookAt._x - _eye._x;
	double zdiff = _lookAt._z - _eye._z;
	return Vector3d(_eye._x - xdiff*amount, _eye._y, _eye._z - zdiff*amount);
}