#include "ParticleEngine.h"

typedef vector<Particle*>::iterator pIter;

/*-------------------------------------------------------------------
* constructor - intialize the position and direction. other variable are presets
---------------------------------------------------------------------*/
ParticleEngine::ParticleEngine(const Vector3d &pos, const Vector3d &dir, const int numParticles){
	position().set(pos);
	_direction.set(dir);
	_iMaxParticles = numParticles;
	_iBatchRelease = 2; // how many particles to release at a time
	_iReleased = 0;

	_texNum = getTexture("particle.bmp", true);

	_particleMemory = operator new(sizeof(Particle) * _iMaxParticles); //grab memory
	_memory = static_cast< Particle* >(_particleMemory); // cast it so we can increment

	//make the particles
	for(int i=0; i<_iMaxParticles; i++){
		Particle * ptP = new (_memory++) Particle(position(), setParticleVelocity(), 10.0f); // make the particle
		_particles.push_back(ptP);
	}
}

/*-----------------------------------------------------
* destructor - delete all the particles created
-------------------------------------------------------*/
ParticleEngine::~ParticleEngine(){
	for(pIter i=_particles.begin(); i!= _particles.end();){
		(*i)->~Particle();
		i = _particles.erase(i);
	}
	delete _particleMemory;
}

/*------------------------------------------------------------------------
* update the particle engine - release more particles, revive dead ones
--------------------------------------------------------------------------*/
void ParticleEngine::update(){
	//release some more particles
	if(_iReleased < _iMaxParticles)
		releaseParticles();

	//check to see if any need to be reset
	for(int i=0; i<_iReleased; i++)
		if(!_particles[i]->isAlive()) resetParticle(_particles[i]);
}

/*---------------------------------------------
* release a set number of particles
-----------------------------------------------*/
void ParticleEngine::releaseParticles(){
	//find out how many to release
	int max = (_iReleased + _iBatchRelease) > _iMaxParticles ? _iMaxParticles : _iReleased + _iBatchRelease;
	for(int i=_iReleased; i<max; i++)
		_iReleased++;
}

/*---------------------------------------------------------------
* reset the particle to a default position and set it off again
-----------------------------------------------------------------*/
void ParticleEngine::resetParticle(Particle * ptParticle){
	ptParticle->reset(position(), setParticleVelocity()); //reset the particle
	ptParticle->setAlive(true); // bring it back to life
}

/*------------------------------------------------------------
* move the particle engine to a new position and normal
--------------------------------------------------------------*/
void ParticleEngine::move(const Vector3d &pos, const Vector3d &dir){
	position().set(pos);
	_direction.set(dir);
}

/*-----------------------------------------------------------
* the draw function - applies the texture to the particle and
* calls the draw function for all of them
-------------------------------------------------------------*/
void ParticleEngine::draw() const {
	glDepthMask(GL_FALSE); // stop writing to the depth buffer so we don't get z buffer 
						   // contension with other particles
						   // but it'll still work with other geometry

	glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE); // enable blending

	glBindTexture(GL_TEXTURE_2D, _texNum); //apply the texture

	for(int i=0; i<_iReleased; i++){
		if(_particles[i]->isAlive()){
			_particles[i]->update(); // update the particle
			_particles[i]->draw(); // draw the particles
		}
	}

	glDisable(GL_BLEND);// disable blending
	glDisable(GL_TEXTURE_2D);

	glDepthMask(GL_TRUE); // enable writing to the depth buffer again
}