#include "PhysicsIF.h"
#include "ObjectMeshManager.h" // include it here rather than the .h so it'll work
#include "Player.h" // include the player here rather than the .h so it'll work
#include "Gun.h" // include the gun here rather than the .h so it'll work
#include "Pipe.h"

// define an iterator to go through the list of static objects that
// ObjectMeshManager manages
typedef vector<PhysicsObject*>::iterator pIter;
typedef vector<Object*>::iterator oIter;

/*----------------------------------------------------------------
* get the next true move for the player - ie if collisions change pos
* work off a 'future' Vector3d position - ie, where the player will be
* NOTE: nextPos is the position of the players eye, not player himself
*
* @returns		a Vector3d for the new position of the eye
------------------------------------------------------------------------*/
Vector3d PhysicsIF::getNextTrueMove(const Player * ptPlayer, const Vector3d & nextEyePos, const bool isGravityChecking){
	//hold the nextEyePos variables in case we need them
	double finalx = nextEyePos._x, finaly = nextEyePos._y, finalz = nextEyePos._z;

	//has the player collided with something
	bool bHasCollided = false;

	float fPlayerHeight = ptPlayer->fHeight(); // store the player's height

	for (pIter envObj = ObjectMeshManager::get()->oAllPhyObjects().begin(); envObj!=ObjectMeshManager::get()->oAllPhyObjects().end(); envObj++) {
		//first check for a collision on the y axis
		//basically we take the max between the two objects minusYBound
		//and the min between the two object plusYBound
		// and if the max is lower than the min, a collision on the y axis has occurred
		//
		// NOTE: player's reg point is taken relative to the ground, not his center
		double Ay = max(nextEyePos._y - fPlayerHeight, (*envObj)->getMinusYBound());
		double By = min(nextEyePos._y, (*envObj)->getPlusYBound());
		if(Ay < By){
			// a collision on the y axis has occured
			// now check the x axis using the same method as above
			double dHalfW = (double)ptPlayer->fWidth() * 0.5; // hold half the player's width
			double Ax = max(nextEyePos._x - dHalfW, (*envObj)->getMinusXBound());
			double Bx = min(nextEyePos._x + dHalfW, (*envObj)->getPlusXBound());
			if(Ax < Bx){
				// a collision on the x axis has occured
				// now check the z axis using the same method as above
				double dHalfD = (double)ptPlayer->fDepth() * 0.5; // hold half the player's depth
				double Az = max(nextEyePos._z - dHalfD, (*envObj)->getMinusZBound());
				double Bz = min(nextEyePos._z + dHalfD, (*envObj)->getPlusZBound());
				if(Az < Bz){
					// a collision on the z axis has also occurred
					// we've collided with something we can't step onto, so adjust the position accordingly
					// find out which side of the object we're colliding with
					// this will allow us to keep moving along one axis while being restricted on another
					double dXpos = ptPlayer->xPos();
					double dZpos = ptPlayer->zPos();
					double dYpos = ptPlayer->yPos();
					//store the bounds of the envObj bounding box (don't need minusY, as it's only used once)
					double dPlusXBound = (*envObj)->getPlusXBound();
					double dPlusYBound = (*envObj)->getPlusYBound();
					double dPlusZBound = (*envObj)->getPlusZBound();
					double dMinusXBound = (*envObj)->getMinusXBound();
					double dMinusZBound = (*envObj)->getMinusZBound();
					//NOTE: we check with the players current pos, not the 'future' one. this is to cover
					// the event that the player is already standing on a step, whereby taking the future pos
					// would return the front side (or similar) as the closest, and not the top, meaning that
					// the player would fall to the object below
					double dLeftSide = abs(dPlusXBound - (dXpos - dHalfW));
					double dRightSide = abs(dMinusXBound - (dXpos + dHalfW));
					double dFrontSide = abs(dPlusZBound - (dZpos - dHalfD));
					double dBackSide = abs(dMinusZBound - (dZpos + dHalfD));
					double dTopSide = abs(dPlusYBound - dYpos);
					double dBottomSide = abs((*envObj)->getMinusYBound() - (dYpos + fPlayerHeight));
					//find the smallest - ie the side of the object that you're colliding with
					double dSmallest = min(min(min(dLeftSide, dRightSide), min(dFrontSide, dBackSide)), min(dTopSide, dBottomSide));
					// if gravity is checking
					if(isGravityChecking){
						if(dSmallest == dTopSide){ // if we're colliding with the top side
							double tempHeight = dPlusYBound + 0.001 + fPlayerHeight;
							if(finaly < tempHeight) finaly = tempHeight; // ensure that we fall on the highest height
							bHasCollided = true;
						}
					} else {
						// if the players _y pos + his stepheight is greater than the upper bounds of the object
						// then that means that he can walk on that object, so move him up
						if(nextEyePos._y - fPlayerHeight + ptPlayer->stepHeight() > dPlusYBound){
							finaly = dPlusYBound + 0.001 + fPlayerHeight; // move the eye pos to the upper boundary
							bHasCollided  = true;											   // of the object, plus the player's height
						} else {
							if(dSmallest == dLeftSide){
								finalx = dPlusXBound + dHalfW; // set the eye x pos to the plusXBound of the object plus dHalfW
							} else if (dSmallest == dRightSide){
								finalx = dMinusXBound - dHalfW; // set the eye x pos to the minusXBound of the object minus dHalfW
							} else if (dSmallest == dFrontSide){
								finalz = dPlusZBound + dHalfD; // set the eye z pos to the plusZBound of the object plus dHalfD
							} else if (dSmallest == dBackSide){
								finalz = dMinusZBound - dHalfD; // set the eye z pos to the minusXBound of the object minus dHalfD
							}
							finaly = nextEyePos._y; // keep the finaly the same ( this is so when we come up to stairs, the player can't climb them
													// front the side - as he'll be able to climb the first one, and so on thereafter. this way if he
													// only collides with the first one, he can walk on it, but if he collides with any of the others
													// in the mean time, then he's at the side, so reset the finaly value
							bHasCollided = true; //set the has collided bool
						}
					} // end of if(isGranvityChecking)
				} // end of if(Az < Bz)
			} // end of if(Ax < Bx)
		} // end of if(Ay < By)
	} // end of loop

	if(!bHasCollided) return nextEyePos; // we' haven't collided with anything, return the eye position unchanged
	else return Vector3d(finalx, finaly, finalz); // we've hit something, so return the changed Vector3d
}

/*------------------------------------------------------------------------------
* check to see if the bullet that the player fired hit anything. bullet's start pos
* is the player's eye, direction is the players _bullet vector. use plane-line
* collision to see if the player hit anything
--------------------------------------------------------------------------------*/
bool PhysicsIF::checkBulletCollision(const Gun *ptGun, const Vector3d &gunPos, const Vector3d &bulletDir, Vector3d &hitPos, Vector3d &hitNormal){
	double dist = ptGun->fFarPlane();
	bool hit = false;
	double lamda; // lamda to hold the time of intersection after bullet was fired

	PhysicsObject * hitObj; // to hold the hit object so we can call onShot()

	// run through the physicsObjects
	for (pIter envObj = ObjectMeshManager::get()->oAllPhyObjects().begin(); envObj!=ObjectMeshManager::get()->oAllPhyObjects().end(); envObj++) {
		if((*envObj)->getID() == "pipe"){
			Vector3d tempHitPos, tempHitNormal; // create tempVector3d's to hold the hitPos and hitNormal
			bool hitPipe = checkBulletCylinder(*envObj, gunPos, bulletDir, lamda, tempHitPos, tempHitNormal);
			if(hitPipe){
				if(lamda < dist) {
					dist = lamda;
					hitPos.set(tempHitPos);
					hitNormal.set(tempHitNormal);
					hitObj = *envObj;
					hit = true;
				}
			}
		} else {
			// hold the normal vector3ds
			const Vector3d * pNormals[6] = {(*envObj)->getMinusXNormal(), (*envObj)->getMinusYNormal(), (*envObj)->getMinusZNormal(),
											(*envObj)->getPlusXNormal(), (*envObj)->getPlusYNormal(), (*envObj)->getPlusZNormal()};
			// hold the point vector3ds
			const Vector3d * pPoints[6] = {(*envObj)->getMinusXPoint(), (*envObj)->getMinusYPoint(), (*envObj)->getMinusZPoint(),
											(*envObj)->getPlusXPoint(), (*envObj)->getPlusYPoint(), (*envObj)->getPlusZPoint()};

			for(int side=0; side < 6; side ++){
				double dotProduct = bulletDir.Dot(*pNormals[side]); // dot product between plane normal and ray direction
				
				// determine if ray is parallel to plane
				if ((dotProduct < NEAR_ZERO) && (dotProduct > -NEAR_ZERO)) continue;

				lamda = (pNormals[side]->Dot(*pPoints[side] - gunPos))/dotProduct;	// find dist to collision


				if (lamda < -NEAR_ZERO) continue; // determine if collision is behind start point

				Vector3d pointOnRay = gunPos + (bulletDir * lamda); // get the point where it intersects plane

				//check to see if the point is inside the object bounding box, and not on it's 'infinite' plane
				if(checkPointInsideBox(*envObj, pointOnRay) && !(*envObj)->isBroken()){
					if(lamda < dist) { // if this is a nearer collision than the previous one
						dist = lamda; // hold the distance
						hitPos.set(pointOnRay); // store the hit position
						hitNormal.set(*pNormals[side]); // store the normals
						hitObj = *envObj;
						hit = true;
					}
				}
			}
		}
	}
	//call onShot()
	if(hit) hitObj->onShot(hitPos, hitNormal);

	return hit ? true : false;
}

/*---------------------------------------------------------------------------------
* check to see if a Vector3d point is inside or on an objects bounding box
-----------------------------------------------------------------------------------*/
bool PhysicsIF::checkPointInsideBox(const PhysicsObject *envObj, const Vector3d &point){
	//check the point with the bounds of the box
	if(point._x >= envObj->getMinusXBound() - NEAR_ZERO && point._x <= envObj->getPlusXBound() + NEAR_ZERO && 
		point._y >= envObj->getMinusYBound() - NEAR_ZERO && point._y <= envObj->getPlusYBound() + NEAR_ZERO &&
		point._z >= envObj->getMinusZBound() - NEAR_ZERO && point._z <= envObj->getPlusZBound() + NEAR_ZERO)
		return true;
	else 
		return false;
}

/*--------------------------------------------------------------------------------------------
* check the bullet against a cylinder. uses a different equation (courtesy of 
* http://nehe.gamedev.net/data/lessons/lesson.asp?lesson=30) so it's sent to a seperate function
----------------------------------------------------------------------------------------------*/
bool PhysicsIF::checkBulletCylinder(PhysicsObject *envObj, const Vector3d &gunPos, const Vector3d &bulletDir, double &lamda, Vector3d &hitPos, Vector3d &hitNormal){
	//first cast the Object pointer to a Pipe pointer
	Pipe * pPipe = dynamic_cast< Pipe *>(envObj);

	Vector3d RC = gunPos.Subtract(pPipe->position());
	Vector3d normal = bulletDir.Cross(pPipe->axis());

	double normalLength = normal.Length();

	// if parallel
	if((normalLength < NEAR_ZERO) && (normalLength > -NEAR_ZERO)) return false;

	normal.Normalise(); // normalize the normal vector

	double d = fabs(RC.Dot(normal)); // get the distance

	if(d <= pPipe->radius()){ // there's been a collision
		Vector3d temp = RC.Cross(pPipe->axis());
		double t = -temp.Dot(normal)/normalLength;

		temp = normal.Cross(pPipe->axis());
		temp.Normalise();

		double s = fabs(sqrt(pPipe->radius()*pPipe->radius() - d*d) / bulletDir.Dot(temp));

		double in = t - s; // where the ray entered
		double out = t + s; // where the ray left

		//return the smallest positive number
		if(in < -NEAR_ZERO){
			if(out < -NEAR_ZERO) return false;
			else lamda = out;
		} else if(out < -NEAR_ZERO) {
			lamda = in;
		} else {
			if(in < out) lamda = in;
			else lamda = out;
		}

		hitPos = gunPos + bulletDir * lamda; // get the hitPosition
		Vector3d HB = hitPos - pPipe->position();
		hitNormal = HB - pPipe->axis()*(HB.Dot(pPipe->axis())); //get the hitNormal
		hitNormal.Normalise(); // normalize it

		return true;
	}
    
	return false;
}