#include "Player.h"

//NOTE: players position is relative to the bottom of the player

/*-------------------------------------------------------------------------------------
* init the player with a camera and pos - pos, fov, aspect, and farPlane all come from the View class
---------------------------------------------------------------------------------------*/
void Player::create(const Vector3d& pos, const float fov, const float aspect, const float farPlane){
	position().set(pos);
	setupCamera(fov, aspect, farPlane);
}

/*-------------------------------------------------------------------------------------
* init the player with a camera and pos - x, y, z, fov, aspect, and farPlane all come from the View class
---------------------------------------------------------------------------------------*/
void Player::create(const float x, const float y, const float z, const float fov, const float aspect, const float farPlane){
	position().set(x, y, z);
	setupCamera(fov, aspect, farPlane);
}

/*----------------------------------
* set up the player's camera
------------------------------------*/
void Player::setupCamera(const float fov, const float aspect, const float farPlane){
	_camera.setCameraShape(fov, aspect, 0.1, farPlane);
	_camera.setCameraPos(_eye); // camera pos is the player's pos plus height
	_camera.turn(0.0f, 0.0f);
}

/*------------------------
* show the view from the players eye(s)
--------------------------*/
void Player::showView(){
	_camera.setMatrix();
}

/*----------------------------------------------------------------------------
* move the player, and thus the camera - gets deltaTime from the View class
------------------------------------------------------------------------------*/
void Player::move(const float deltaTime){
	float moveAmount = _fSpeed * _iDirection * deltaTime;
	//find the next move - check the physics with the camera's next potential move
	Vector3d nextMove = getNextTrueMove(this, _camera.calcNextMove(moveAmount), false);
	updatePlayer(nextMove);
}

/*---------------------------------------
* set the center and eye Vector3d's
-----------------------------------------*/
void Player::setEyeAndCenter(const Vector3d &pos){
	_center.set(pos._x, pos._y + fHeight() * 0.5, pos._z); //center
	_eye.set(pos._x, pos._y + fHeight(), pos._z); // eye
}

/*------------------------------------------------------------------------------------------------------
* update the various things after moving - ie: camera, position, eye, center, boundingbox, and gun
--------------------------------------------------------------------------------------------------------*/
void Player::updatePlayer(const Vector3d & nextPos){
	_camera.move(nextPos); // move the camera
	position().set(nextPos._x, nextPos._y - fHeight(), nextPos._z);
	setEyeAndCenter(position()); // update the eye and center
	moveBoundingBox(_center); // move the players bounding box
	_gun.updatePosition(_eye); // update the gun position
}

/*-----------------------------------------------------------------------------------------------------
* turn the player, and thus the camera - xAngle and yAngle come from OnMouseMove() in the View  class
-------------------------------------------------------------------------------------------------------*/
void Player::turn(const float xAngle, const float yAngle){
	_camera.turn(xAngle, yAngle);
}

/*----------------------------------------------------------
* change the players height to the new newYPos - update the camera
* bounding box, and eye and center vectors - TESTING ONLY
----------------------------------------------------------*/
void Player::changeYPos(const double newYPos){
	position()._y = newYPos;
	setEyeAndCenter(position());
	_camera.setCameraPos(_eye); // camera pos is the player's pos plus height
	_camera.updatePosition(); // update the camera _lookAt and _up vectors
	moveBoundingBox(_center); // move the bounding box
}

/*------------------
* apply gravity
--------------------*/
void Player::applyGravity(const float deltaTime){
	double tempY = _eye._y - (1.0 * _fSpeed * deltaTime); // next eye._y based on gravity
	Vector3d nextMove = getNextTrueMove(this, Vector3d(xPos(), tempY, zPos()), true);
	updatePlayer(nextMove);
}

/*----------------------
* the read function
------------------------*/
void Player::read(istream &fin){
	char buffer[256], ch;
	float w, h, d, sh; // width, height, depth, and stepHeight
	int numBullets; // number of bullets

	fin.get(buffer, sizeof(buffer), '=');
	fin >> ch >> _fFov; // read in the fov

	fin.get(buffer, sizeof(buffer), '=');
	fin >> ch >> _fAspectRatioX; // read in the aspectRatioX

	fin.get(buffer, sizeof(buffer), '=');
	fin >> ch >> _fAspectRatioY; // read in the aspectRatioY
	
	fin.get(buffer, sizeof(buffer), '=');
	fin >> ch >> _fFarPlane; // read in the farPlane
	
	fin.get(buffer, sizeof(buffer), '=');
	fin >> ch >> w; // read in the width
	fWidth(w); // set it
	
	fin.get(buffer, sizeof(buffer), '=');
	fin >> ch >> h; // read in the height
	fHeight(h); // set it
	
	fin.get(buffer, sizeof(buffer), '=');
	fin >> ch >> d; // read in the depth
	fDepth(d); // set it
	
	fin.get(buffer, sizeof(buffer), '=');
	fin >> ch >> _fSpeed; // read in the speed

	fin.get(buffer, sizeof(buffer), '=');
	fin >> ch >> numBullets; // read in the number of bullets

	fin.get(buffer, sizeof(buffer), '=');
	fin >> ch >> sh; // read in the stepHeight
	stepHeight(sh); //set it

	fin.get(buffer, sizeof(buffer), '=');
	fin >> ch >> position(); // read in the position

	//set up the center and eye pos
	setEyeAndCenter(position());

	//set up the camera
	setupCamera(_fFov, _fAspectRatioX/_fAspectRatioY, _fFarPlane);

	//set up the bounding box
	setBoundingBox(fWidth(), fHeight(), fDepth(), _center);

	//set up the player's gun
	_gun.create(_eye, numBullets, _fFarPlane);
}

/*---------------------------------------------
* the write function - stream out the data
-----------------------------------------------*/
ostream& Player::write(ostream &out){
	out << "Player {"
		 << "\n\tfov= " << _fFov
		 << "\n\taspectRatioX= " << _fAspectRatioX
		 << "\n\taspectRatioY= " << _fAspectRatioY
		 << "\n\tfarPlane= " << _fFarPlane
		 << "\n\twidth= " << fWidth()
		 << "\n\theight= " << fHeight()
		 << "\n\tdepth= " << fDepth()
		 << "\n\tspeed= " << _fSpeed
		 << "\n\tbullets= " << _gun.iTotalBullets()
		 << "\n\tstepHeight= " << stepHeight()
		 << "\n\tposition= " << position()
		 << "\n}";
	return out;
}