#pragma once

#include "PhysicsObject.h"
#include "Camera.h"
#include "Gun.h"
#include <fstream>

using namespace std;

class Player : public PhysicsObject{
private:
	float _fSpeed, _fStepHeight; // speed, step height (for camera pos) - step height used to determine the height of objects
								 // that the player can step on
	float _fFov, _fAspectRatioX, _fAspectRatioY, _fFarPlane; // fov, aspect ratio, and far plane for the camera
	bool _bIsMoving; // is the player currently moving
	int _iDirection; // -1 for forward, 1 for backwards
	Camera _camera; // camera used for displaying the scene
	Vector3d _center, _eye; // center position used for the bounding box, eye position for holding the eye

	Gun _gun; //gun object for the player

	void setupCamera(const float fov, const float aspect, const float farPlane); // set up the player's camera

public:
	//constructor to initialize the basics
	Player(): _iDirection(0), _bIsMoving(0){};

	//init the player with a camera and pos - pos, fov, aspect, and farPlane all come from the View class
	// used when creating a player directly, not from streaming from a file
	void create(const Vector3d & pos, const float fov, const float aspect, const float farPlane);

	//init the player with a camera and pos - x, y, z, fov, aspect, and farPlane all come from the View class
	// used when creating a player directly, not from streaming from a file
	void create(const float x, const float y, const float z, const float fov, const float aspect, const float farPlane);

	//show the view from the players eye(s)
	void showView();

	// move the player, and thus the camera - gets deltaTime from the View class
	void move(const float deltaTime);

	// set the center and eye Vector3d's
	void setEyeAndCenter(const Vector3d &pos);

	//update the various things after moving - ie: camera, position, eye, center, boundingbox, and gun
	void updatePlayer(const Vector3d & nextPos);

	//turn the player, and thus the camera - xAngle and yAngle come from OnMouseMove() in the View  class
	void turn(const float xAngle, const float yAngle);

	//change the players height to the new newYPos - update the camera
	// bounding box, and eye and center vectors - TESTING ONLY
	void changeYPos(const double newYPos);

	// apply gravity - change pos, if there's nothing in the way
	void applyGravity(const float deltaTime);

	/*--------- setters and getters --------------------------------*/

	//set and get the _bIsMoving bool
	void isMoving(const bool isMoving) { _bIsMoving = isMoving; };
	bool isMoving() const { return _bIsMoving; };

	//set the shooting bool on the player's gun
	void setIsShooting(const bool b) { _gun.bIsShooting(b); };//_bIsShooting = b; };

	//set the _iDirection int
	void direction(const int dir) { _iDirection = dir; };

	//get and set the step height
	float stepHeight() const { return _fStepHeight; };
	void stepHeight(const float sh) { _fStepHeight = sh; };

	float getFarPlane() const { return _fFarPlane; }; // return the far plane

	int getBulletsLeft() const { return _gun.iTotalBullets; };

	/*--------------------------------------------------------------*/

	// receive the bullet direction vector3d from the view class, which works it out
	// based on the mousex, and mousey coords. pass it on to the gun to fire
	void shoot(const Vector3d & bulletDir){ _gun.shoot(bulletDir); };

	//the draw function - (NOT NEEDED FOR PLAYER?)
	void draw() const { return; };

	//the read function
	void read(istream &fin);
	
	//the write function
	ostream& write(ostream &out);
};

// overloaded write operator
inline ostream &operator<<(ostream &out, Player &player) { player.write(out); return out; }