#include "Skybox.h"

/*------------------------------------------------------
* create the Skybox - initialize the vars and the planes
--------------------------------------------------------*/
void Skybox::create(const float fWidth, const float fHeight, const float fDepth,
					const GLuint iFrontTexture, const GLuint iBackTexture, const GLuint iLeftTexture,
					const GLuint iRightTexture, const GLuint iTopTexture, const GLuint iBottomTexture,
					const bool repeatTexture){
	//initialize the vars etc
	_fWidth = fWidth;
	_fHeight = fHeight;
	_fDepth = fDepth;
	_iFrontTexture = iFrontTexture;
	_iBackTexture = iBackTexture;
	_iLeftTexture = iLeftTexture;
	_iRightTexture = iRightTexture;
	_iTopTexture = iTopTexture;
	_iBottomTexture = iBottomTexture;

	//initialize the planes, with each plane taking its width and height
	//depending on where it is in the cube
	_pFrontFace.create(_fWidth, _fHeight, _iFrontTexture, repeatTexture); // front face
	_pBackFace.create(_fWidth, _fHeight, _iBackTexture, repeatTexture); // back face
	_pLeftFace.create(_fDepth, _fHeight, _iLeftTexture, repeatTexture); // left face
	_pRightFace.create(_fDepth, _fHeight, _iRightTexture, repeatTexture); // right face
	_pTopFace.create(_fWidth, _fDepth, _iTopTexture, repeatTexture); // top face
	_pBottomFace.create(_fWidth, _fDepth, _iBottomTexture, repeatTexture); //bottom face
}

/*----------------------------------------------------------
* builds the Skybox. moves each face into place, then calls
* the Plane::draw() method to draw it
------------------------------------------------------------*/
void Skybox::buildSkybox() const{
	float fHalfW = _fWidth * 0.5f;
	float fHalfH = _fHeight * 0.5f;
	float fHalfD = _fDepth * 0.5f;

	//translate and rotate each face into position before calling the draw method on it
	//NOTE - the cubes origin is the bottom left hand corner to make it easier to position,
	//		 so translate and rotate accordingly
	glPushMatrix();
		glTranslatef(fHalfW, fHalfH, 0.0);
		_pFrontFace.draw(); // front face
	glPopMatrix();

	glPushMatrix();
		glTranslatef(fHalfW, fHalfH, _fDepth);
		glRotatef(180, 0.0f, 1.0f, 0.0f);
		_pBackFace.draw(); // back face
	glPopMatrix();

	glPushMatrix();
		glTranslatef(0.0, fHalfH, fHalfD);
		glRotatef(90, 0.0f, 1.0f, 0.0f);
		_pLeftFace.draw(); // left face
	glPopMatrix();

	glPushMatrix();
		glTranslatef(_fWidth, fHalfH, fHalfD);
		glRotatef(-90, 0.0f, 1.0f, 0.0f);
		_pRightFace.draw(); // right face
	glPopMatrix();

	glPushMatrix();
		glTranslatef(fHalfW, 0.0, fHalfD);
		glRotatef(-90, 1.0f, 0.0f, 0.0f);
		_pTopFace.draw(); // top face
	glPopMatrix();

	glPushMatrix();
		glTranslatef(fHalfW, _fHeight, fHalfD);
		glRotatef(90, 1.0f, 0.0f, 0.0f);
		_pBottomFace.draw(); // bottom face
	glPopMatrix();
}

/*-------------------------------------
* the draw function - call buildSkybox
---------------------------------------*/
void Skybox::draw() const{
	glPushMatrix();
		glTranslated(xPos(), yPos(), zPos());
		buildSkybox();
	glPopMatrix();
}

/*-------------------------------------------------------------------------
* the read function - gets passed the istream from the ObjectMeshManager
---------------------------------------------------------------------------*/
void Skybox::read(istream &fin){
	char buffer[256], ch;
	
	// create two array, one for the texture strings for each side
	// and one for the corresponding number variables in this object
	// (need to make arrays of pointers to the variables themselves)
	string * sStreamTexNames[] = {&_sFrontTexture, &_sBackTexture, &_sLeftTexture, &_sRightTexture, &_sTopTexture, &_sBottomTexture};
	GLuint * iTexNumArray[] = {&_iFrontTexture, &_iBackTexture, &_iLeftTexture, &_iRightTexture, &_iTopTexture, &_iBottomTexture};

	fin.get(buffer, sizeof(buffer), '=');
	fin >> ch >> _fWidth; // read in the width

	fin.get(buffer, sizeof(buffer), '=');
	fin >> ch >> _fHeight; // read in the height

	fin.get(buffer, sizeof(buffer), '=');
	fin >> ch >> _fDepth; // read in the depth

	fin.get(buffer, sizeof(buffer), '=');
	fin >> ch >> position(); // read in the position

	for(int i=0; i<6; i++){
		fin.get(buffer, sizeof(buffer), '=');
		fin >> ch >> ch >> *sStreamTexNames[i] >> ch; // read in the six textures
		*iTexNumArray[i] = getTexture(*sStreamTexNames[i]); // assign the texture number
	}

	/*--------------------------------------------------------*/
	string tempStr;
	fin.get(buffer, sizeof(buffer), '=');
	fin >> ch >> ch >> tempStr >> ch; // read in the id
	setID(tempStr); // set it
	/*--------------------------------------------------------*/

	create(_fWidth, _fHeight, _fDepth,
				  _iFrontTexture, _iBackTexture, _iLeftTexture, _iRightTexture, _iTopTexture, _iBottomTexture,
				  false); // create the cuboid
}

/*---------------------------------------------
* the write function - stream out the data
-----------------------------------------------*/
ostream& Skybox::write(ostream &out){
	out << "Skybox {"
		 << "\n\twidth= " << _fWidth
		 << "\n\theight= " << _fHeight
		 << "\n\tdepth= " << _fDepth
		 << "\n\tposition= " << position()
		 << "\n\tfrontTexture= \"" << _sFrontTexture << " \""
		 << "\n\tbackTexture= \"" << _sBackTexture << " \""
		 << "\n\tleftTexture= \"" << _sLeftTexture << " \""
		 << "\n\trightTexture= \"" << _sRightTexture << " \""
		 << "\n\ttopTexture= \"" << _sTopTexture << " \""
		 << "\n\tbottomTexture= \"" << _sBottomTexture << " \""
		 << "\n\tid= \"" << getID() << " \""
		 << "\n}" << endl;
	return out;
}