#include "TextureManager.h"

//initialize the instance to 0
TextureManager * TextureManager::_instance = 0;

/*------------------------------------------------------
* destructor - run through the _texturesLoaded map
* and delete the entries
--------------------------------------------------------*/
TextureManager::~TextureManager(){
	//run through the texturesLoaded Map, and delete the entries
	_texturesLoaded.erase(_texturesLoaded.begin(), _texturesLoaded.end());
}

/*--------------------------------------------------------------------
* search through the _texturesLoaded map and compare the names
* there with the string passed (first). if one is found, then pass back the
* corresponding texture number (second). if there
* isn't one found, then return -1 to indicate that the texture needs
* to be created
----------------------------------------------------------------------*/
int TextureManager::searchLoaded(const string &name){
	for(mIter i = _texturesLoaded.begin(); i!= _texturesLoaded.end(); i++){
		if(name.compare((*i).first) == 0) return (*i).second; // compare the key value (the name string)
															// with the name passed. if they match, return
															// the second value (GLuint texture number)
	}
	return -1;
}

/*-------------------------------------------------------------------
* create a texture. load in the image specified by 'name' into the 
* _image Image object build mipmaps from
* the image, then release the resource. add the newly created texture
* name and number to the _texturesLoaded map, and return the number
----------------------------------------------------------------------*/
GLuint TextureManager::create(const string &name){
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR_MIPMAP_LINEAR);

	bool succeed = _image.Load(name.c_str()); // load the image
	if(succeed){
		GLuint i;
		
		glGenTextures(1, &i); //generate the texture - pass the address of i

		// build 2D mip-maps from image
		glBindTexture(GL_TEXTURE_2D, i);
		_image.gluBuild2DMipmaps();
		_image.Free(); // free the resource

		//create a new pair, name and number, and insert it into the texturesLoaded Map
		Pair * ptr = new Pair(name, i);
		_texturesLoaded.insert(*ptr);
		delete ptr; // delete the pointer

		return(i); // return the texture number
	} else
		return 0;
}

/*-------------------------------------------------------------------
* create an alpha mask. load in the image specified by 'name' into the 
* _image Image object. get GXBase to convert it to an 8bit alpha image
* glTexImage2d() to use it to modify the alpha values of the polygon
* it's mapped to. add the newly created textrue name and number to the
* _texturesLoaded map, then return the number
----------------------------------------------------------------------*/
GLuint TextureManager::createAlphaMask(const string &name){
	_image.Load(name.c_str()); // load the image
	_image.SetFormat(Image::ALPHA); // ask gxBase to convert it to an alpha mask
	GLuint i;
	
	glGenTextures(1, &i); //generate the texture - pass the address of i

	// build 2D mip-maps from image
	glBindTexture(GL_TEXTURE_2D, i);
	//define how the texture is to be accessed (no mipmaps)
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

	_image.glTexImage2D(); // specify that the texture should be used to modify the alpha values for the
					// polygon
	_image.Free(); // free the resource

	//create a new pair, name and number, and insert it into the texturesLoaded Map
	Pair * ptr = new Pair(name, i);
	_texturesLoaded.insert(*ptr);
	delete ptr; // delete the pointer

	return(i); // return the texture number
}

/*-----------------------------------------------------------------------
* the getTexture function. first compare the name passed with the names
* of the textures already loaded in the _sTextureNamesLoaded list. if
* it's found, then return the number of that texture from the _iTextureNumbers
* list. if its not, then create a texture, and return the number
--------------------------------------------------------------------------*/
GLuint TextureManager::getTexture(const string &name, bool isAlphaMask){
	int n = searchLoaded(name); //search to see if we've already created the texture
								//return -1 if false
	if(n == -1){
		return isAlphaMask ? createAlphaMask(name) : create(name); //create and return the number
	} else {
		return n; //return the number
	}
}

/*----------------------------------------------------------------------------------------
* save the screen to a texture - provide the name of the object that's doing the saving.
* if the object has created a texture before, then find the number and overwrite it,
* otherwise create one and save it
------------------------------------------------------------------------------------------*/
void TextureManager::screenToTexture(Image& image, const string &name){
	int n = searchLoaded(name); // search to see if we've already created the texture
								// return -1 if false
	GLuint i = 0;
	if(n != -1) i = n;

	if(n == -1){
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR_MIPMAP_LINEAR);
	
		glGenTextures(1, &i); //generate the texture - pass the address of i
	}

	// build 2D mip-maps from image
	glBindTexture(GL_TEXTURE_2D, i);
	image.gluBuild2DMipmaps();

	image.Free(); // free the resource

	if(n == -1){
		//create a new pair, name and number, and insert it into the texturesLoaded Map
		Pair * ptr = new Pair(name, i);
		_texturesLoaded.insert(*ptr);
		delete ptr; // delete the pointer
	}
}