#include "Vector3d.h"

/*--------------------------------------------------------
* Default constructor. Initialises vector to zero.
----------------------------------------------------------*/
Vector3d::Vector3d() {
	_x = 0.0;
	_y = 0.0;
	_z = 0.0;
}

/*------------------------------------------------------------------------
* Construct from double. Assigns double value to all elements of vector.
--------------------------------------------------------------------------*/
Vector3d::Vector3d(double d){
	_x = d;
	_y = d;
	_z = d;
}

/*-------------------------------------------------------------------------
* Construct from three doubles. Assigns the three parameters to the
* X,Y,Z elements of the vector.
---------------------------------------------------------------------------*/
Vector3d::Vector3d(double X, double Y, double Z){
	_x = X;
	_y = Y;
	_z = Z;
}

/*-----------------------------------
* set an existing Vector3d
-------------------------------------*/
void Vector3d::set(const Vector3d &R){
	_x = R._x;
	_y = R._y;
	_z = R._z;
}

/*-------------------------------------------
* set an existing Vector3d using doubles
---------------------------------------------*/
void Vector3d::set(const double x, const double y, const double z){
	_x = x;
	_y = y;
	_z = z;
}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------

/*----------------------------------------------------------------------------
* Exact equality. Returns true if both vectors are exactly equal. It is
* not good practice to use this operator. Use IsEqual() instead.
------------------------------------------------------------------------------*/
bool Vector3d::operator == (const Vector3d &V) const {
	return (_x == V._x) && (_y == V._y) && (_z == V._z);
}

/*-----------------------------------------------------------------------------
* Inequality operator. Returns true if the vectors are not exactly equal.
* It is not good practice to use this operator. Use !IsEqual() instead.
-------------------------------------------------------------------------------*/
bool Vector3d::operator != (const Vector3d &V) const {
	return !(*this == V);
}

/*-------------------------------------------------------------
* overloaded addition operator - calls the Add function
---------------------------------------------------------------*/
Vector3d Vector3d::operator +(const Vector3d &R) const {
	return this->Add(R);
}

/*----------------------------------------------------------------
* overloaded subtraction operator - calls the Subtract function
------------------------------------------------------------------*/
Vector3d Vector3d::operator - (const Vector3d &R) const {
	return this->Subtract(R);
}

/*--------------------------------------------------------------
* overloaded * operator - calls the Dot function
----------------------------------------------------------------*/
double Vector3d::operator * (const Vector3d &R) const{
	return this->Dot(R);
}

/*-------------------------------------------------------------
* overloaded * operator with a scalar - calls the MultScalar function
---------------------------------------------------------------*/
Vector3d Vector3d::operator *(double d) const{
	return this->MultScalar(d);
}

/*-------------------------------------------------------
* overloaded ^ operator - calls the Cross function
---------------------------------------------------------*/
Vector3d Vector3d::operator ^ (const Vector3d &R) const{
	return this->Cross(R);
}

/*------------------------------------------------------------
* overloaded += operator. changes the vector that calls it
--------------------------------------------------------------*/
void Vector3d::operator += (const Vector3d &R){
	_x += R._x;
	_y += R._y;
	_z += R._z;
}

/*------------------------------------------------------------
* overloaded -= operator. changes the vector that calls it
--------------------------------------------------------------*/
void Vector3d::operator -= (const Vector3d &R){
	_x -= R._x;
	_y -= R._y;
	_z -= R._z;
}

/*-------------------------------------------------
* overloaded unary - operator. reverses the vector
---------------------------------------------------*/
Vector3d Vector3d::operator -() {
	return Vector3d(-_x, -_y, -_z);
}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------

/*---------------------------------------------------------------------------
* Vector addition. The code A = L.Add(R) is equivalent to A=L+R
-----------------------------------------------------------------------------*/
Vector3d Vector3d::Add(const Vector3d &R) const {
	return Vector3d(_x + R._x, _y + R._y, _z + R._z);	
}

/*-------------------------------------------------------------------------
* Vector subtraction. The code A = L.Subtract(R) is equivalent to A=L-R
---------------------------------------------------------------------------*/
Vector3d Vector3d::Subtract(const Vector3d &R) const {
	return Vector3d(_x - R._x, _y - R._y, _z - R._z);	
}

/*------------------------------------------------------
* Vector multiplication by a scalar
--------------------------------------------------------*/
Vector3d Vector3d::MultScalar(const double d) const{
	return Vector3d(_x*d, _y*d, _z*d);
}

/*----------------------------------------------------------------------------
* Vector dot (scalar) product. produces a scaler. the code D = A.Dot(B) is
* equivilant to AxBx + AyBy + AzBz
------------------------------------------------------------------------------*/
double Vector3d::Dot(const Vector3d &R) const {
	return ((_x*R._x) + (_y*R._y) + (_z*R._z));
}

/*----------------------------------------------------------------------------
* Vector cross (vector) product. gives a Vector that is perpendicular to
* the two Vectors used. the code A = D.Cross(C) is equivilant to
* (AyBz - AzBy)i + (AzBx - AxBz)j + (AxBy - AyBz)k
------------------------------------------------------------------------------*/
Vector3d Vector3d::Cross(const Vector3d &R) const {
	return Vector3d((_y*R._z - _z*R._y), (_z*R._x - _x*R._z), (_x*R._y - _y*R._x));
}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------

/*-----------------------------------------------------
* Distance between two vectors.
-------------------------------------------------------*/
double Vector3d::Distance  (const Vector3d &V) const {
	return (double)sqrt( DistanceSqr(V) );
}

/*-----------------------------------------------------------------
* Squared distance between two vectors. When comparing distances, it is
* generally faster to compare the squared distance to avoid the square
* root calculation.
-------------------------------------------------------------------------*/
double Vector3d::DistanceSqr (const Vector3d &V) const {
	return Subtract(V).LengthSqr();
}

/*----------------------------------------------------------------
* Length of vector. This returns the length (magnitude) of the vector.
-----------------------------------------------------------------------*/
double Vector3d::Length  () const {
	return (double)sqrt( LengthSqr() );
}

/*-------------------------------------------------------------------------
* Squared length of vector. This returns the squared length of the
* vector (squared magnitude). When comparing length, it is generally
* faster to compare the squared length to avoid the square root
* calculation.
---------------------------------------------------------------------------*/
double Vector3d::LengthSqr () const {
	return (_x*_x + _y*_y + _z*_z);
}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------

/*--------------------------------------------------------------------
* Normalise a vector in place. If the vector is zero magnitude, then
* the result is a zero vector.
----------------------------------------------------------------------*/
const Vector3d & Vector3d::Normalise() {
	return Normalise(*this);
}

/*------------------------------------------------------------------------
* Normalise a vector. The operation L.Normalise(R) replaces L with
* the normalised (unit length) version of R. It is equivalent to
* L=R.Normalise() but with less overhead, since a temporary object
* is not returned.
--------------------------------------------------------------------------*/
const Vector3d & Vector3d::Normalise( const Vector3d &V ) {
	double r = V.Length();
	if (r!=0.0) // guard against divide by zero
		return *this = Vector3d(V._x/r, V._y/r, V._z/r); // normalise and return
	else
		return *this;
}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------

/*-------------------------------------------------------------------
* Vector is near zero. Returns true if a vector is very close to zero.
* The default tolerance can be overidden.
---------------------------------------------------------------------*/
bool Vector3d::IsZero  (double Tolerance) const {
	return (LengthSqr() <= Tolerance*Tolerance);
}

/*----------------------------------------------------------------------
* Vector is near unit-length. Returns true if vector is very close to
* unit length. The default tolerance can be overidden.
------------------------------------------------------------------------*/
bool Vector3d::IsUnit (double Tolerance) const {
	return fabs(Length() - 1.0) <= Tolerance;
}

/*-------------------------------------------------------------------------------------------
* Vector this is near equal to V. Returns true if vector this and V are very close to equal.
* The default tolerance can be overidden.
---------------------------------------------------------------------------------------------*/
bool Vector3d::IsEqual(const Vector3d &V, double Tolerance) const {
	return Subtract(V).IsZero(Tolerance);
}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

/*------------------------------------------------------
* write facilitator - used for streaming out
--------------------------------------------------------*/
void Vector3d::write(ostream &out) const {
	out << "(" << _x
		<< ", " << _y
		<< ", " << _z << ")";
}

/*----------------------------------------------------
* read facilitator - used for streaming in
------------------------------------------------------*/
void Vector3d::read(istream &in) {
	char ch;
	in >> ch >> _x
		>> ch >> _y
		>> ch >> _z >> ch;
}