#pragma once

#include <iostream>
#include <cmath>
using namespace std;

#define V3_EPSILON 1.0E-8


class Vector3d {
public:
	double _x;
	double _y;
	double _z;

public:

	// Default constructor. Initialises vector to zero.
	Vector3d();

	// Construct from double. Assigns double value to all elements of vector.
	Vector3d(double);

	// Construct from three doubles. Assigns the three parameters to the
	// X,Y,Z elements of the vector.
	Vector3d(double, double, double);

	// set an existing Vector3d
	void set(const Vector3d &);

	// set an existing Vector3d using doubles
	void set(const double, const double, const double);

	/*-------------------------------------------------------------------*/

	// Exact equality. Returns true if both vectors are exactly equal. It is
	// not good practice to use this operator. Use IsEqual() instead.
	bool operator == (const Vector3d &) const;

	// Inequality operator. Returns true if the vectors are not exactly equal.
	// It is not good practice to use this operator. Use !IsEqual() instead.
	bool operator != (const Vector3d &) const;

	// overloaded addition operator - calls the Add function
	Vector3d operator + (const Vector3d &) const;

	// overloaded subtraction operator - calls the Subtract function
	Vector3d operator - (const Vector3d &) const;

	// overloaded * operator - calls the Dot function
	double operator * (const Vector3d &) const;

	// overloaded * operator with a scalar - calls the MultScalar function
	Vector3d operator * (double d) const;

	// second overloaded * operator with a scalar - this multiplies a scaler by a Vector3d
	//Vector3d operator * (const double &d, const Vector3d &R) const;

	// overloaded ^ operator - calls the Cross function
	Vector3d operator ^ (const Vector3d &) const;

	// overloaded += operator. changes the vector that calls it
	void operator += (const Vector3d &);

	// overloaded -= operator. changes the vector that calls it
	void operator -= (const Vector3d &);

	// overloaded unary - operator. reverses the vector
	Vector3d operator - ();

	//-------------------------------------------------------------------------

	// Vector addition. The code A = L.Add(R) is equivalent to A=L+R
	Vector3d Add(const Vector3d &R ) const;

	// Vector subtraction. The code A = L.Subtract(R) is equivalent to A=L-R
	Vector3d Subtract(const Vector3d &R ) const;

	// Vector multiplication by a scalar
	Vector3d MultScalar(const double d) const;
	
	// Vector dot (scalar) product. produces a scaler. the code D = A.Dot(B) is
	// equivilant to AxBx + AyBy + AzBz
	double Dot(const Vector3d &R) const;

	// Vector cross (vector) product. gives a Vector that is perpendicular to
	// the two Vectors used. the code A = D.Cross(C) is equivilant to
	// (AyBz - AzBy)i + (AzBx - AxBz)j + (AxBy - AyBz)k
	Vector3d Cross(const Vector3d &R) const;

	//-------------------------------------------------------------------------

	// Distance between two vectors.
	double Distance (const Vector3d &V) const;

	// Squared distance between two vectors. When comparing distances, it is
	// generally faster to compare the squared distance to avoid the square
	// root calculation.
	double DistanceSqr (const Vector3d &V) const;

	// Length of vector. This returns the length (magnitude) of the vector.
	double Length () const;

	// Squared length of vector. This returns the squared length of the
	// vector (squared magnitude). When comparing length, it is generally
	// faster to compare the squared length to avoid the square root
	// calculation.
	double LengthSqr () const;

	//-------------------------------------------------------------------------

	// Normalise a vector in place. If the vector is zero magnitude, then
	// the result is a zero vector.
	const Vector3d & Normalise();

	// Normalise a vector. The operation L.Normalise(R) replaces L with
	// the normalised (unit length) version of R. It is equivalent to
	// L=R.Normalise() but with less overhead, since a temporary object
	// is not returned.
	const Vector3d & Normalise( const Vector3d &V );


	//-------------------------------------------------------------------------

	// Vector is near zero. Returns true if a vector is very close to zero.
	// The default tolerance can be overidden.
	bool IsZero  (double Tolerance = V3_EPSILON) const;

	// Vector is near unit-length. Returns true if vector is very close to
	// unit length. The default tolerance can be overidden.
	bool IsUnit  (double Tolerance = V3_EPSILON) const;

	// Vector this is near equal to V. Returns true if vector this and V are very close to equal.
	// The default tolerance can be overidden.
	bool IsEqual(const Vector3d &V, double Tolerance = V3_EPSILON) const;

	//-----------------------------------------------------------------------

	// write facilitator - used for streaming out
	void write(ostream &out) const;

	// read facilitator - used for streaming in
	void read(istream &in);
};

//------------------------------------------------------

// auxiliary functions

// stream out
inline ostream& operator << (ostream &out, const Vector3d &R) { R.write(out); return out; }

// stream in
inline istream& operator >> (istream &in, Vector3d &R) { R.read(in); return in; }

//multiply a scalar by a Vector3d - THIS DOESN'T WORK???????????????????
//Vector3d Vector3d::operator * (const double &d, const Vector3d &R) const { return R.MultScalar(d); };